@@ -11,6 +11,7 @@ from __future__ import annotations
 
 import collections.abc as cabc
 import copy
+import io
 import logging
 import operator
 import typing as t
@@ -34,7 +35,7 @@ from .conftest import (
 )
 
 TEST_SNAPSHOT_PATH = TEST_DATA_PATH / "snapshots"
-TEST_SNAPSHOT: list[actiontypes.TrackerSnapshot] = yaml.safe_load(
+TEST_SNAPSHOT: actiontypes.Snapshot = yaml.safe_load(
     (TEST_SNAPSHOT_PATH / "snapshot.yaml").read_text(encoding="utf-8")
 )
 TEST_SNAPSHOT_1 = yaml.safe_load(
@@ -99,7 +100,9 @@ class TestTrackerChangeInit(ActionsTest):
         del tconfig["capella-uuid"]  # type:ignore[misc]
 
         with pytest.raises(actiontypes.InvalidTrackerConfig):
-            self.tracker_change(clean_model, TEST_SNAPSHOT[0], tconfig)
+            self.tracker_change(
+                clean_model, TEST_SNAPSHOT["modules"][0], tconfig
+            )
 
     def test_init_on_missing_module_raises_MissingRequirementsModule(
         self, clean_model: capellambse.MelodyModel
@@ -113,13 +116,13 @@ class TestTrackerChangeInit(ActionsTest):
         """
         del clean_model.la.requirement_modules[0]
 
-        with pytest.raises(change.MissingRequirementsModule):
-            self.tracker_change(clean_model, TEST_SNAPSHOT[0])
+        with pytest.raises(change.MissingCapellaModule):
+            self.tracker_change(clean_model, TEST_SNAPSHOT["modules"][0])
 
     def test_init_on_missing_module_id_raises_InvalidSnapshotModule(
         self, clean_model: capellambse.MelodyModel
     ) -> None:
-        snapshot = copy.deepcopy(TEST_SNAPSHOT[0])
+        snapshot = copy.deepcopy(TEST_SNAPSHOT["modules"][0])
         del snapshot["id"]  # type: ignore[misc]
 
         with pytest.raises(actiontypes.InvalidSnapshotModule):
@@ -129,7 +132,7 @@ class TestTrackerChangeInit(ActionsTest):
 class TestCreateActions(ActionsTest):
     """UnitTests for all methods requesting creations."""
 
-    tracker = TEST_SNAPSHOT[0]
+    tracker = TEST_SNAPSHOT["modules"][0]
     titem = tracker["items"][0]
     tracker_change_creations = TEST_MODULE_CHANGE[0]["extend"]
 
@@ -205,7 +208,7 @@ class TestCreateActions(ActionsTest):
         first_child = titem["children"][0]
         first_child["attributes"][attr] = faulty_value  # type:ignore[index]
         message_end = (
-            f"Invalid field found: {key} '{faulty_value}' for '{attr}'"
+            f"Invalid field found: {key} {faulty_value!r} for {attr!r}"
         )
 
         with caplog.at_level(logging.ERROR):
@@ -227,12 +230,12 @@ class TestCreateActions(ActionsTest):
             ] = faulty_value  # type:ignore[index]
             messages.append(
                 "Invalid workitem 'REQ-002'. "
-                f"Invalid field found: {key} '{faulty_value}' for '{attr}'"
+                f"Invalid field found: {key} {faulty_value!r} for {attr!r}"
             )
 
-        change = self.tracker_change(clean_model, tracker, gather_logs=True)
+        tchange = self.tracker_change(clean_model, tracker, gather_logs=True)
 
-        assert change.errors == messages
+        assert tchange.errors == messages
 
     def test_faulty_data_types_log_InvalidAttributeDefinition_as_error(
         self,
@@ -262,9 +265,45 @@ class TestCreateActions(ActionsTest):
             "data_type": "Not-Defined",
         }
 
-        change = self.tracker_change(clean_model, tracker, gather_logs=True)
+        tchange = self.tracker_change(clean_model, tracker, gather_logs=True)
 
-        assert change.errors == [INVALID_ATTR_DEF_ERROR_MSG]
+        assert tchange.errors == [INVALID_ATTR_DEF_ERROR_MSG]
+
+    def test_requirements_with_empty_children_are_rendered_as_folders(
+        self, clean_model: capellambse.MelodyModel
+    ) -> None:
+        tracker = copy.deepcopy(self.tracker)
+        tracker["items"][0]["children"][1]["children"][0]["children"] = []
+
+        change_set = self.tracker_change(
+            clean_model, tracker, gather_logs=True
+        )
+
+        assert change_set and (tchange := change_set.actions[0]["extend"])
+        assert (
+            tchange["folders"][0]["folders"][0]["folders"][0]["identifier"]
+            == "REQ-004"
+        )
+
+    def test_enum_value_long_name_collision_produces_no_Unfulffilled_Promises(
+        self, clean_model: capellambse.MelodyModel
+    ) -> None:
+        tracker = copy.deepcopy(self.tracker)
+        tracker["data_types"]["new"] = tracker[  # type: ignore[index]
+            "data_types"
+        ]["Type"]
+        tracker["requirement_types"]["system_requirement"][  # type: ignore
+            "attributes"
+        ]["new"] = {"type": "Enum"}
+        req_item = tracker["items"][0]["children"][0]
+        req_item["attributes"]["new"] = ["Functional"]  # type: ignore[index]
+
+        change_set = self.tracker_change(
+            clean_model, tracker, gather_logs=True
+        )
+
+        yml = decl.dump(change_set.actions)
+        decl.apply(clean_model, io.StringIO(yml))
 
     @pytest.mark.integtest
     def test_calculate_change_sets(
@@ -272,7 +311,7 @@ class TestCreateActions(ActionsTest):
     ) -> None:
         """Test ``ChangeSet`` on clean model for first migration run."""
         change_set, errors = calculate_change_set(
-            clean_model, TEST_CONFIG["modules"][0], TEST_SNAPSHOT[0]
+            clean_model, TEST_CONFIG["modules"][0], TEST_SNAPSHOT["modules"][0]
         )
 
         assert not errors
@@ -282,7 +321,7 @@ class TestCreateActions(ActionsTest):
 class TestModActions(ActionsTest):
     """Tests all methods that request modifications."""
 
-    tracker = TEST_SNAPSHOT_1[0]
+    tracker = TEST_SNAPSHOT_1["modules"][0]
     titem = tracker["items"][0]
 
     ENUM_DATA_TYPE_MODS = TEST_MODULE_CHANGE_1[:2]
@@ -295,7 +334,9 @@ class TestModActions(ActionsTest):
     ) -> None:
         r"""Test producing ``ModAction``\ s for EnumDataTypeDefinitions."""
         snapshot = copy.deepcopy(self.tracker)
-        snapshot["requirement_types"] = TEST_SNAPSHOT[0]["requirement_types"]
+        snapshot["requirement_types"] = TEST_SNAPSHOT["modules"][0][
+            "requirement_types"
+        ]
         snapshot["items"] = []
 
         tchange = self.tracker_change(migration_model, snapshot)
@@ -308,8 +349,8 @@ class TestModActions(ActionsTest):
     ) -> None:
         r"""Test producing ``ModAction``\ s for RequirementTypes."""
         snapshot = copy.deepcopy(self.tracker)
-        snapshot["data_types"] = TEST_SNAPSHOT[0]["data_types"]
-        snapshot["items"] = TEST_SNAPSHOT[0]["items"]
+        snapshot["data_types"] = TEST_SNAPSHOT["modules"][0]["data_types"]
+        snapshot["items"] = TEST_SNAPSHOT["modules"][0]["items"]
 
         tchange = self.tracker_change(migration_model, snapshot)
 
@@ -338,7 +379,7 @@ class TestModActions(ActionsTest):
         first_child = titem["children"][0]
         first_child["attributes"][attr] = faulty_value
         message_end = (
-            f"Invalid field found: {key} '{faulty_value}' for '{attr}'"
+            f"Invalid field found: {key} {faulty_value!r} for {attr!r}"
         )
 
         with caplog.at_level(logging.ERROR):
@@ -346,13 +387,113 @@ class TestModActions(ActionsTest):
 
         assert caplog.messages[0].endswith(message_end)
 
+    def test_faulty_data_types_log_InvalidAttributeDefinition_as_error(
+        self,
+        migration_model: capellambse.MelodyModel,
+        caplog: pytest.LogCaptureFixture,
+    ) -> None:
+        tracker = copy.deepcopy(self.tracker)
+        reqtype = tracker["requirement_types"]["system_requirement"]
+        reqtype["attributes"]["Not-Defined"] = {  # type: ignore[call-overload]
+            "type": "Enum",
+            "data_type": "Not-Defined",
+        }
+
+        with caplog.at_level(logging.ERROR):
+            self.tracker_change(migration_model, tracker, gather_logs=False)
+
+        assert caplog.messages[0].endswith(INVALID_ATTR_DEF_ERROR_MSG)
+
+    def test_faulty_simple_attributes_log_AttributeError(
+        self,
+        migration_model: capellambse.MelodyModel,
+        caplog: pytest.LogCaptureFixture,
+    ) -> None:
+        """Test logging ``AttributeError`` on faulty simple attributes."""
+        tracker = copy.deepcopy(self.tracker)
+        titem = tracker["items"][0]
+        titem["imagination"] = 1
+        message_end = (
+            "Invalid module 'project/space/example title'. Invalid "
+            "workitem 'REQ-001'. imagination isn't defined on Folder"
+        )
+
+        with caplog.at_level(logging.ERROR):
+            self.tracker_change(migration_model, tracker, gather_logs=False)
+
+        assert caplog.messages[0].endswith(message_end)
+
+    def test_InvalidAttributeDefinition_errors_are_gathered(
+        self, migration_model: capellambse.MelodyModel
+    ) -> None:
+        """Test faulty field data are gathered in errors."""
+        tracker = copy.deepcopy(self.tracker)
+        reqtype = tracker["requirement_types"]["system_requirement"]
+        reqtype["attributes"]["Not-Defined"] = {  # type: ignore[call-overload]
+            "type": "Enum",
+            "data_type": "Not-Defined",
+        }
+
+        tchange = self.tracker_change(
+            migration_model, tracker, gather_logs=True
+        )
+
+        assert tchange.errors == [INVALID_ATTR_DEF_ERROR_MSG]
+
+    def test_requirements_with_empty_children_are_rendered_as_folders(
+        self, migration_model: capellambse.MelodyModel
+    ) -> None:
+        parent_uuid = "9a9b5a8f-a6ad-4610-9e88-3b5e9c943c19"
+        req_uuid = "163394f5-c1ba-4712-a238-b0b143c66aed"
+        tracker = copy.deepcopy(self.tracker)
+        tracker["items"][1]["children"][0]["children"] = []
+
+        change_set = self.tracker_change(
+            migration_model, tracker, gather_logs=True
+        )
+
+        assert change_set and change_set.actions
+        for action in change_set.actions:
+            if action["parent"].uuid == parent_uuid:
+                tchange = action
+                break
+        else:
+            assert False, "Did not find any action"
+
+        folder_extensions = tchange["extend"]["folders"]
+        requirement_deletions = tchange["delete"]["requirements"]
+        assert len(folder_extensions) == 1
+        assert folder_extensions[0]["identifier"] == "REQ-004"
+        assert len(requirement_deletions) == 1
+        assert requirement_deletions[0].uuid == req_uuid
+
+    def test_enum_value_long_name_collision_produces_no_Unfulffilled_Promises(
+        self, migration_model: capellambse.MelodyModel
+    ) -> None:
+        tracker = copy.deepcopy(self.tracker)
+        tracker["data_types"]["new"] = tracker["data_types"]["Type"]
+        tracker["requirement_types"]["system_requirement"]["attributes"][
+            "new"
+        ] = {"type": "Enum"}
+        req_item = tracker["items"][0]["children"][0]
+        req_item["attributes"]["new"] = ["Functional"]
+
+        change_set = self.tracker_change(
+            migration_model, tracker, gather_logs=True
+        )
+
+        yml = decl.dump(change_set.actions)
+        decl.apply(migration_model, io.StringIO(yml))
+
     @pytest.mark.integtest
     def test_calculate_change_sets(
         self, migration_model: capellambse.MelodyModel
     ) -> None:
         """Test ChangeSet on clean model for first migration run."""
         change_set, errors = calculate_change_set(
-            migration_model, TEST_CONFIG["modules"][0], TEST_SNAPSHOT_1[0]
+            migration_model,
+            TEST_CONFIG["modules"][0],
+            TEST_SNAPSHOT_1["modules"][0],
         )
 
         assert not errors
@@ -362,7 +503,7 @@ class TestModActions(ActionsTest):
 class TestDeleteActions(ActionsTest):
     """Test all methods that request deletions."""
 
-    tracker = TEST_SNAPSHOT_2[0]
+    tracker = TEST_SNAPSHOT_2["modules"][0]
     titem = tracker["items"][0]
 
     REQ_TYPE_FOLDER_DEL = TEST_MODULE_CHANGE_2[0]
@@ -402,7 +543,9 @@ class TestDeleteActions(ActionsTest):
     ) -> None:
         """Test that EnumDataTypeDefinitions are deleted."""
         snapshot = copy.deepcopy(self.tracker)
-        snapshot["requirement_types"] = TEST_SNAPSHOT_1[0]["requirement_types"]
+        snapshot["requirement_types"] = TEST_SNAPSHOT_1["modules"][0][
+            "requirement_types"
+        ]
         snapshot["items"] = []
         data_type_del = copy.deepcopy(self.REQ_TYPE_FOLDER_DEL)
         del data_type_del["delete"]["requirement_types"]
@@ -419,7 +562,7 @@ class TestDeleteActions(ActionsTest):
     ) -> None:
         """Test that RequirementTypes are deleted."""
         snapshot = copy.deepcopy(self.tracker)
-        snapshot["data_types"] = TEST_SNAPSHOT_1[0]["data_types"]
+        snapshot["data_types"] = TEST_SNAPSHOT_1["modules"][0]["data_types"]
         snapshot["items"] = []
         req_type_del = copy.deepcopy(self.REQ_TYPE_FOLDER_DEL)
         del req_type_del["delete"]["data_type_definitions"]
@@ -435,8 +578,10 @@ class TestDeleteActions(ActionsTest):
     ) -> None:
         """Test that RequirementsModActions are produced."""
         snapshot = copy.deepcopy(self.tracker)
-        snapshot["data_types"] = TEST_SNAPSHOT_1[0]["data_types"]
-        snapshot["requirement_types"] = TEST_SNAPSHOT_1[0]["requirement_types"]
+        snapshot["data_types"] = TEST_SNAPSHOT_1["modules"][0]["data_types"]
+        snapshot["requirement_types"] = TEST_SNAPSHOT_1["modules"][0][
+            "requirement_types"
+        ]
         requirement_del = self.resolve_ChangeSet(deletion_model, self.REQ_DEL)
 
         tchange = self.tracker_change(deletion_model, snapshot)
@@ -457,7 +602,9 @@ class TestDeleteActions(ActionsTest):
         )
 
         change_set, errors = calculate_change_set(
-            deletion_model, TEST_CONFIG["modules"][0], TEST_SNAPSHOT_2[0]
+            deletion_model,
+            TEST_CONFIG["modules"][0],
+            TEST_SNAPSHOT_2["modules"][0],
         )
 
         assert not errors
@@ -479,13 +626,16 @@ class TestCalculateChangeSet(ActionsTest):
         config = copy.deepcopy(TEST_CONFIG["modules"][0])
         del config["capella-uuid"]  # type:ignore[misc]
         message = (
-            "The given module configuration is missing 'UUID' of the target "
+            "The given module configuration is missing UUID of the target "
             "RequirementsModule"
         )
 
         with caplog.at_level(logging.ERROR):
             calculate_change_set(
-                clean_model, config, TEST_SNAPSHOT[0], gather_logs=False
+                clean_model,
+                config,
+                TEST_SNAPSHOT["modules"][0],
+                gather_logs=False,
             )
 
         assert caplog.messages[0] == f"{self.SKIP_MESSAGE}. {message}"
@@ -505,7 +655,10 @@ class TestCalculateChangeSet(ActionsTest):
 
         with caplog.at_level(logging.ERROR):
             calculate_change_set(
-                clean_model, tconfig, TEST_SNAPSHOT[0], gather_logs=False
+                clean_model,
+                tconfig,
+                TEST_SNAPSHOT["modules"][0],
+                gather_logs=False,
             )
 
         assert caplog.messages[0] == f"{self.SKIP_MESSAGE}. {message}"
@@ -515,12 +668,12 @@ class TestCalculateChangeSet(ActionsTest):
         clean_model: capellambse.MelodyModel,
         caplog: pytest.LogCaptureFixture,
     ) -> None:
-        snapshot = copy.deepcopy(TEST_SNAPSHOT[0])
+        snapshot = copy.deepcopy(TEST_SNAPSHOT["modules"][0])
         del snapshot["id"]  # type: ignore[misc]
         tconfig = TEST_CONFIG["modules"][0]
         message = (
             "Skipping module: MISSING ID. "
-            "In the snapshot the module is missing an 'id' key"
+            "In the snapshot the module is missing an id key"
         )
 
         with caplog.at_level(logging.ERROR):
@@ -537,24 +690,62 @@ class TestCalculateChangeSet(ActionsTest):
         config = copy.deepcopy(TEST_CONFIG["modules"][0])
         del config["capella-uuid"]  # type:ignore[misc]
         message = (
-            "The given module configuration is missing 'UUID' of the target "
+            "The given module configuration is missing UUID of the target "
             "RequirementsModule"
         )
 
         _, errors = calculate_change_set(
-            clean_model, config, TEST_SNAPSHOT[0], gather_logs=True
+            clean_model, config, TEST_SNAPSHOT["modules"][0], gather_logs=True
         )
 
         assert errors[0].startswith(self.SKIP_MESSAGE)
         assert message in errors[0]
 
+    def test_forced_calculation_produces_change_set_on_AttributeDefinition_error(
+        self, clean_model: capellambse.MelodyModel
+    ) -> None:
+        """Test that an invalid AttributeDefinition will not prohibit."""
+        snapshot = copy.deepcopy(TEST_SNAPSHOT["modules"][0])
+        missing_enumdt = "Release"
+        del snapshot["data_types"][missing_enumdt]  # type: ignore[attr-defined]
+        tconfig = TEST_CONFIG["modules"][0]
+        message = (
+            "In RequirementType 'System Requirement': Invalid "
+            "AttributeDefinitionEnumeration found: 'Release'. Missing its "
+            "datatype definition in `data_types`.\n"
+            "Invalid workitem 'REQ-002'. Invalid field found: 'Release'. "
+            "Missing its datatype definition in `data_types`."
+        )
+
+        change_sets, errors = calculate_change_set(
+            clean_model, tconfig, snapshot, gather_logs=True, force=True
+        )
+
+        assert (change_set := change_sets[0])
+        for rtfolder in change_set["extend"]["requirement_types_folders"]:
+            for req_type in rtfolder["requirement_types"]:
+                for attr_def in req_type["attribute_definitions"]:
+                    assert attr_def["long_name"] != missing_enumdt
+
+        folder = change_set["extend"]["folders"][0]
+        for folder in folder["folders"]:
+            for attr_value in folder.get("attributes", []):
+                assert (
+                    missing_enumdt not in attr_value["definition"].identifier
+                )
+
+        assert errors[0].startswith(
+            "Encountered error(s) in 'project/space/example title'"
+        )
+        assert message in errors[0]
+
     def test_snapshot_errors_from_ChangeSet_calculation_are_gathered(
         self, clean_model: capellambse.MelodyModel
     ) -> None:
-        snapshot = copy.deepcopy(TEST_SNAPSHOT[0])
+        snapshot = copy.deepcopy(TEST_SNAPSHOT["modules"][0])
         titem = snapshot["items"][0]
         first_child = titem["children"][0]
-        titem["attributes"]["Test"] = 1  # type: ignore[index]
+        titem["attributes"] = {"Test": 1}  # type: ignore[index]
         messages = [
             "Invalid workitem 'REQ-001'. "
             "Invalid field found: field name 'Test' not defined in "
@@ -566,9 +757,10 @@ class TestCalculateChangeSet(ActionsTest):
             ] = faulty_value  # type:ignore[index]
             messages.append(
                 "Invalid workitem 'REQ-002'. "
-                + f"Invalid field found: {key} '{faulty_value}' for '{attr}'"
+                + f"Invalid field found: {key} {faulty_value!r} for {attr!r}"
             )
         del titem["children"][1]["type"]
+        titem["children"][1]["attributes"] = {"Test": 1}
         tconfig = TEST_CONFIG["modules"][0]
         messages.append(
             "Invalid workitem 'REQ-003'. Missing type but attributes found"
